@@ -1,49 +1,13 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QFONT_H
 #define QFONT_H
 
+#include <QtCore/qshareddata.h>
 #include <QtGui/qtguiglobal.h>
 #include <QtGui/qwindowdefs.h>
 #include <QtCore/qstring.h>
-#include <QtCore/qsharedpointer.h>
 
 
 QT_BEGIN_NAMESPACE
@@ -162,7 +126,8 @@ public:
         HintingPreferenceResolved   = 0x8000,
         StyleNameResolved           = 0x10000,
         FamiliesResolved            = 0x20000,
-        AllPropertiesResolved       = 0x3ffff
+        FeaturesResolved            = 0x40000,
+        AllPropertiesResolved       = 0x7ffff
     };
     Q_ENUM(ResolveProperties)
 
@@ -170,15 +135,12 @@ public:
 
     QFont(const QString &family, int pointSize = -1, int weight = -1, bool italic = false);
     explicit QFont(const QStringList &families, int pointSize = -1, int weight = -1, bool italic = false);
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    QFont(const QFont &font, QPaintDevice *pd);
-#endif
     QFont(const QFont &font, const QPaintDevice *pd);
     QFont(const QFont &font);
     ~QFont();
 
-    void swap(QFont &other)
-    { qSwap(d, other.d); qSwap(resolve_mask, other.resolve_mask); }
+    void swap(QFont &other) noexcept
+    { d.swap(other.d); std::swap(resolve_mask, other.resolve_mask); }
 
     QString family() const;
     void setFamily(const QString &);
@@ -245,6 +207,20 @@ public:
     void setHintingPreference(HintingPreference hintingPreference);
     HintingPreference hintingPreference() const;
 
+    // Note: The following set of APIs are preliminary and may change in future releases
+    void setFeature(const char *feature, quint32 value);
+    void setFeature(quint32 tag, quint32 value);
+    void unsetFeature(quint32 tag);
+    void unsetFeature(const char *feature);
+    quint32 featureValue(quint32 tag) const;
+    bool isFeatureSet(quint32 tag) const;
+    QList<quint32> featureTags() const;
+    void clearFeatures();
+
+    static QByteArray tagToString(quint32 tag);
+    static quint32 stringToTag(const char *tagString);
+    // --
+
     // dupicated from QFontInfo
     bool exactMatch() const;
 
